precision highp float;
precision highp int;

uniform sampler2D u_sTexture0;
uniform sampler2D u_sTexture1;
uniform highp float u_alpha;
uniform highp mat4 u_color_matrix;
varying highp vec2 v_v2TexCoord;
varying highp vec2 v_v2TexCoordScreen;

float blendOverlay(float base, float blend) {
	return base<0.5?(2.0*base*blend):(1.0-2.0*(1.0-base)*(1.0-blend));
}

vec3 blendOverlay(vec3 base, vec3 blend) {
	return vec3(blendOverlay(base.r,blend.r),blendOverlay(base.g,blend.g),blendOverlay(base.b,blend.b));
}

vec3 blendOverlay(vec3 base, vec3 blend, float opacity) {
	return (blendOverlay(base, blend) * opacity + base * (1.0 - opacity));
}

vec3 blendHardLight(vec3 base, vec3 blend) {
	return blendOverlay(blend,base);
}

vec3 blendHardLight(vec3 base, vec3 blend, float opacity) {
	return (blendHardLight(base, blend) * opacity + base * (1.0 - opacity));
}

void main()
{
    highp vec4 col0 = texture2D(u_sTexture0, v_v2TexCoord);
    highp vec4 col1 = texture2D(u_sTexture1, vec2(v_v2TexCoordScreen.x, v_v2TexCoordScreen.y));

    highp vec4 color;
    highp vec4 tCol;
    if ((col1.a > 0.0 && col1.a <1.0) && col0.a == 1.0)
    {
        tCol = vec4(col1.rgb / col1.a, col1.a);
        tCol = tCol * u_color_matrix;
        color.rgb = blendHardLight(col0.rgb, tCol.rgb, tCol.a);
    }
    else if ((col0.a > 0.0 && col0.a <1.0) && col1.a == 1.0)
    {
        tCol = vec4(col0.rgb / col0.a, col0.a);
        tCol = tCol * u_color_matrix;
        color.rgb = blendHardLight(col1.rgb, tCol.rgb, tCol.a);
    }
    else if (col0.a >= col1.a)
        color.rgb = blendHardLight(col0.rgb , col1.rgb , col1.a);
    else //if (col0.a < col1.a)
        color.rgb = blendHardLight(col1.rgb , col0.rgb , col0.a);

    color.a = max(col1.a, col0.a);

    color = color * u_color_matrix;
    color = clamp(color, 0.0, 1.0);

    gl_FragColor = color * u_alpha;
}